"""The property SHV API."""

import collections.abc

import shv

from .abc import SHVAPI


class SHVProperty(SHVAPI):
    """The property node API."""

    def __init__(
        self,
        data: str,
        readonly: bool = False,
        signal: str | bool = False,
        description: str = "",
    ) -> None:
        self._data = data
        self._readonly = readonly
        self._signal = signal
        self._description = description

    @property
    def data(self) -> str:
        """Type of the data property works with."""
        return self._data

    # TODO rpctype

    @property
    def readonly(self) -> bool:
        """If property is read-only."""
        return self._readonly

    @property
    def signal(self) -> str | None:
        """Signal name for the property value change."""
        return (
            self._signal
            if isinstance(self._signal, str)
            else "chng"
            if self._signal
            else None
        )

    @property
    def description(self) -> str:
        """Description of this property."""
        return self._description

    def methods(self) -> collections.abc.Iterator[shv.RpcMethodDesc]:  # noqa: D102
        yield shv.RpcMethodDesc.getter(
            result=self._data, signal=self._signal, description=self._description
        )
        if not self._readonly:
            yield shv.RpcMethodDesc.setter(param=self._data)
